// direct_download_links - Add direct download links
// version 0.2
// 2011-11-14
// Copyright (C) 2011  Antonio Ospite <ospite@studenti.unina.it>
// Released under the GPL license
// http://www.gnu.org/copyleft/gpl.html
//
// --------------------------------------------------------------------
//
// This is a Greasemonkey user script.
//
// To install, you need Greasemonkey: https://addons.mozilla.org/en-US/firefox/addon/748
// Then restart Firefox and revisit this script.
// Under Tools, there will be a new menu item to "Install User Script".
// Accept the default configuration and install.
//
// To uninstall, go to Tools/Manage User Scripts,
// select "Direct Download Links", and click Uninstall.
//
// --------------------------------------------------------------------
//
// ==UserScript==
// @name           Direct Download Links
// @namespace      http://git.ao2.it/GM_direct_download_links.git
// @description    Add direct download links
// @include        http://video.repubblica.it/*
// @include        http://tv.repubblica.it/*
// @include        http://trovacinema.repubblica.it/*
// @include        http://www.kataweb.it/tvzap/*
// @include        http://www.rai.tv/*
// ==/UserScript==
//
// NOTE, for rai.tv to work you need to install a script like:
// http://git.ao2.it/smooth-dl.git/blob_plain/HEAD:/scripts/SilverSpoof.user.js

/*
 * TODO:
 *  - find a way to use the same string as in the @include lines to match the
 *    current window.location. Look for something like GM_testUrl() which builds
 *    the regexp starting from a glob line.
 *  - use xpath instead of regexp like in http://a32.me/2009/11/greasemonkey/
 *  - use jquery, like shown in http://a32.me/2009/11/greasemonkey/
 */

/* Fields supported by the "site" object.
 *
 * Manadatory fields:
 *   pageURL: the URL of the page we are modifying
 *   urlContainer: the element containing the URL to link
 *   urlRegexp: the regular expression for finding the URL, the first
 *              sub-pattern is taken as the URL
 *   linkDest: the element where to place the Direct Download link
 *
 *
 * Optional fields:
 *   onEvent: used to delay the urlRegexp matching to a certain event like
 *            'DOMNodeInserted' useful when the URL is added by some javascript
 *            library. It has two fields:
 *
 *              evt: the event we want to wait for (e.g. 'DOMNodeInserted')
 *
 *              targetElement: the element in the event handler we want the
 *                urlRegexp is performed on.
 *
 */
var supported_sites = [
  {
    locationRegexp: /^http:\/\/video\.repubblica\.it\/.*$/,
    urlContainer: 'contA',
    urlRegexp: /'pcUrl', '((http|mms):\/\/[^']*)'/,
    linkDest: 'contA',
  },
  {
    locationRegexp: /^http:\/\/tv\.repubblica\.it\/.*$/,
    urlContainer: 'boxPlayer',
    urlRegexp: /'pcUrl', '((http|mms):\/\/[^']*)'/,
    linkDest: 'box_embed',
  },
  {
    locationRegexp: /^http:\/\/trovacinema\.repubblica\.it\/.*$/,
    urlContainer: 'col-center',
    urlRegexp: /'flvUrl', '((http|mms):\/\/[^']*)'/,
    linkDest: 'col-center',
  },
  {
    locationRegexp: /^http:\/\/www\.kataweb\.it\/tvzap\/.*$/,
    urlContainer: 'tvzap_video',
    urlRegexp: /'pcUrl', '((http|mms):\/\/[^']*)'/,
    linkDest: 'playerCont',
  },
  {
    locationRegexp: /^http:\/\/www\.rai\.tv\/.*$/,
    urlContainer: 'Player',
    urlRegexp: /mediaUri=(http:\/\/[^,]*)/,
    onEvent: { evt: 'DOMNodeInserted', targetElement: 'object' },
    linkDest: 'Player',
  },
];

/* Apply different rules to different sites */
for (i = 0; i < supported_sites.length; i++) {
  var site = supported_sites[i];

  var result = window.location.href.match(site.locationRegexp);
  if (result) {
    direct_download_link_add(window.location.href, site);
  }
}

/**
 * Add a Direct Download link on the page for the specified URL
 *
 * @param: a 'site' object described above.
 *
 * @return: null on error, true on success
 */
function direct_download_link_add(pageURL, site) {
  var element = document.getElementById(site.urlContainer);
  if (!element) {
    DDL_log('DirectDL (' + site.pageURL  + '): Cannot find the element ' + site.urlContainer + ' containing the URL.');
    return null;
  }

  document.addEventListener('UrlFetched', _add_link, true);

  // This is used for sites adding the URL to the DOM after DOMContentLoaded,
  // for example by some javascript library (like Silverlight.js on rai.tv).
  if (site.onEvent) {
    element.addEventListener(site.onEvent.evt, function(e) {
      if (site.onEvent.targetElement &&
          e.target.tagName.toLowerCase() != site.onEvent.targetElement) {
        DDL_log('DirectDL (' + site.pageURL  + '): skipping element ' + e.target.tagName);
        return;
      }
     _get_URL(site, element);
    }, false);
  }

  _get_URL(site, element);
}

function _get_URL(site, element) {
  var content = element.innerHTML;
  if (!content) {
    DDL_log('DirectDL (' + site.pageURL + '): content is null, cannot find URL.');
    return;
  }

  var matches = content.match(site.urlRegexp);
  if (!matches || matches.length < 2 || !matches[1]) {
      DDL_log('DirectDL (' + site.pageURL + '): URL not found, check the urlRegexp');
      return;
  }
  var URL = matches[1];
  if (!URL) {
    DDL_log('DirectDL (' + site.pageURL + '): cannot get the URL.');
    return;
  }

  var evt = document.createEvent('Event');  
  evt.initEvent('UrlFetched', true, true);  
  evt.site = site;
  evt.URL = URL;
  document.dispatchEvent(evt);
}

function _add_link(e) {
  var site = e.site;
  var URL = e.URL;;

  var links = document.getElementById(site.linkDest);
  if (!links) {
    DDL_log('DirectDl (' + site.pageURL + '): Cannot add the direct download link.');
    return;
  }

  var download_link = document.createElement('a');
  download_link.textContent = 'Direct Link';
  download_link.setAttribute('href', URL);
  var style = 'background-color: white; color: blue;';
  style += ' border: 2px solid red;'
  style += ' float: right; font-size: large;';
  style += ' padding: .5em; margin: 1em;'
  download_link.setAttribute('style', style);

  links.insertBefore(download_link, links.firstChild);
}

function DDL_log(message) {
  var debug = false;
  if (debug) {
    alert(message)
  } else {
    GM_log(message);
  }
}
